from typing import Dict

import requests

from desec_dns_api import UnexpectedError, FieldError, UnauthorizedError, NonEmptyAccountError


def obtain_captcha() -> Dict:
    """
    Gets a captcha from desec API.
    https://desec.readthedocs.io/en/latest/auth/account.html#obtain-a-captcha

    :return:
    """
    r = requests.post('https://desec.io/api/v1/captcha/')

    if r.status_code != 201:
        raise UnexpectedError(f'API returned status { r.status_code }.')

    # {
    #   "id": "f743336e-3515-4be0-9658-70cdfd16a930",
    #   "challenge": "iVBORw0KGgoAAAANSUhEUgAAAKAAAAA8C....fFUvrwAAAABJRU5ErkJggg=="
    # }
    return r.json()


def register(email: str, password: str, captcha_id: str, captcha_solution: str, domain: str = None):
    """
    Registers an account with desec API.
    https://desec.readthedocs.io/en/latest/auth/account.html#register-account

    :param email: Email address for the account
    :param password: Password for the account
    :param captcha_id: ID of the captcha
    :param captcha_solution: Solution to the captcha
    :param domain: DNS zone that should be created with the account
    :return:
    """
    data = {
        'email': email,
        'password': password,
        'captcha': {
            'id': captcha_id,
            'solution': captcha_solution
        }
    }
    if domain:
        data['domain'] = domain

    r = requests.post('https://desec.io/api/v1/auth/', json=data)

    if r.status_code == 400:
        msg = ''
        for field in r.json():
            msg += f'Errors in field "{ field }": { " ".join([e for e in r.json()[field]]) }\n'
            #for error in r.json()[field]:
            #    msg += f' { error }'
        raise FieldError(msg.strip())
    elif r.status_code != 202:
        raise UnexpectedError(f'API returned status { r.status_code }')


def login(email: str, password: str) -> str:
    """
    Login and retrieve auth token.
    https://desec.readthedocs.io/en/latest/auth/account.html#log-in

    :param email: Account email address
    :param password: Account password
    :return: Authentication token
    """
    data = {
        'email': email,
        'password': password
    }

    r = requests.post('https://desec.io/api/v1/auth/login/', json=data)

    if r.status_code == 401:
        raise UnauthorizedError('Authentication failed')
    elif r.status_code == 200:
        return r.json()['token']
    else:
        raise UnexpectedError(f'API returned status { r.status_code }')


def info(token: str) -> Dict:
    """
    Retrieve account info.
    https://desec.readthedocs.io/en/latest/auth/account.html#retrieve-account-information

    :param token: Authentication token
    :return:
    """
    r = requests.get('https://desec.io/api/v1/auth/account/', headers={'Authorization': f'Token { token }'})

    if r.status_code == 401:
        raise UnauthorizedError('Authentication failed')
    elif r.status_code != 200:
        raise UnexpectedError(f'API returned status { r.status_code }')

    # {
    #   "created": "2019-10-16T18:09:17.715702Z",
    #   "email": "youremailaddress@example.com",
    #   "id": "9ab16e5c-805d-4ab1-9030-af3f5a541d47",
    #   "limit_domains": 5
    # }
    return r.json()


def reset_password(email: str, captcha_id: str, captcha_solution: str):
    """
    Request a new password.
    https://desec.readthedocs.io/en/latest/auth/account.html#password-reset

    :param email: Email address of the account
    :param captcha_id: UUID of the solved captcha
    :param captcha_solution: Solution to the solved captcha
    :return:
    """
    data = {
        'email': email,
        'captcha': {
            'id': captcha_id,
            'solution': captcha_solution
        }
    }

    r = requests.post('https://desec.io/api/v1/auth/account/reset-password/', json=data)

    if r.status_code == 400:
        msg = ''
        for field in r.json()['captcha']:
            msg += f'Errors in field "{field}": {" ".join([e for e in r.json()["captcha"][field]])}\n'
            # for error in r.json()[field]:
            #    msg += f' { error }'
        raise FieldError(msg.strip())
    elif r.status_code != 202:
        raise Exception('Unknown error')


def change_email_address(email: str, password: str, new_email: str):
    """
    Request email address change.
    https://desec.readthedocs.io/en/latest/auth/account.html#change-email-address

    :param email: Email address of the account
    :param password: Password of the account
    :param new_email: New email address for the account
    :return:
    """
    data = {
        'email': email,
        'password': password,
        'new_email': new_email
    }

    r = requests.post('https://desec.io/api/v1/auth/account/change-email/', json=data)

    if r.status_code != 202:
        raise Exception('Unknown error')


def delete_account(email: str, password: str):
    """
    Request account deletion.
    https://desec.readthedocs.io/en/latest/auth/account.html#delete-account

    :param email: Email address of the account
    :param password: Password of the account
    :return:
    """
    data = {
        'email': email,
        'password': password
    }

    r = requests.post('https://desec.io/api/v1/auth/account/delete/', json=data)

    if r.status_code == 409:
        raise NonEmptyAccountError('There are still domains in the account')
    elif r.status_code == 403:
        raise UnauthorizedError(r.json()["detail"])
    elif r.status_code != 202:
        raise UnexpectedError(f'API returned status { r.status_code }')


def logout(token: str):
    """
    Logout and remove token from account.
    https://desec.readthedocs.io/en/latest/auth/account.html#log-out

    :param token: The token to remove.
    :return:
    """
    r = requests.post('https://desec.io/api/v1/auth/logout/', headers={'Authorization': f'Token { token }'})

    if r.status_code != 204:
        raise UnexpectedError(f'API returned status { r.status_code }')
