from typing import Dict, List

import requests


def create_domain(token: str, name: str) -> Dict:
    """
    Create a new domain.
    https://desec.readthedocs.io/en/latest/dns/domains.html#creating-a-domain

    :param token: The token to authenticate
    :param name: The domain to create
    :return:
    """
    r = requests.post('https://desec.io/api/v1/domains/', headers={'Authorization': f'Token { token }'})

    if r.status_code == 400:
        raise Exception('Improper request')
    elif r.status_code == 403:
        raise Exception('Maximum number of domains for your account reached')
    elif r.status_code != 201:
        raise Exception('Unknown error')

    # {
    #     "created": "2018-09-18T16:36:16.510368Z",
    #     "keys": [
    #         {
    #             "dnskey": "257 3 13 WFRl60...",
    #             "ds": [
    #                 "6006 13 1 8581e9...",
    #                 "6006 13 2 f34b75...",
    #                 "6006 13 3 dfb325...",
    #                 "6006 13 4 2fdcf8..."
    #             ],
    #             "flags": 257,
    #             "keytype": "csk"
    #         },
    #         ...
    #     ],
    #     "minimum_ttl": 3600,
    #     "name": "example.com",
    #     "published": "2018-09-18T17:21:38.348112Z"
    # }
    return r.json()


def list_domains(token: str) -> List[Dict]:
    """
    List all domains.
    https://desec.readthedocs.io/en/latest/dns/domains.html#listing-domains

    :param token: The token to authenticate
    :return:
    """
    # TODO: handle pagination: https://desec.readthedocs.io/en/latest/dns/rrsets.html#pagination
    r = requests.get('https://desec.io/api/v1/domains/', headers={'Authorization': f'Token { token }'})

    if r.status_code != 200:
        raise Exception('Unknown error')

    return r.json()


def get_domain(token: str, name: str) -> Dict:
    """
    Retrieve a domain with a specific name.
    https://desec.readthedocs.io/en/latest/dns/domains.html#retrieving-a-specific-domain

    :param token: The token to authenticate
    :param name: Name of the domain
    :return:
    """
    r = requests.get(f'https://desec.io/api/v1/domains/{ name }/', headers={'Authorization': f'Token { token }'})

    if r.status_code == 404:
        raise Exception('Domain not found in account')
    if r.status_code != 200:
        raise Exception('Unknown error')

    return r.json()


def delete_domain(token: str, name: str):
    """
    Delete a domain.
    https://desec.readthedocs.io/en/latest/dns/domains.html#deleting-a-domain

    :param token: The token to authenticate
    :param name: The domain to delete
    :return:
    """
    r = requests.delete(f'https://desec.io/api/v1/domains/{ name }/', headers={'Authorization': f'Token { token }'})

    if r.status_code != 204:
        raise Exception('Unknown error')
