from typing import List, Dict

import requests


def create_rrset(token: str, name: str, subname: str, type: str, ttl: int, records: List[str]) -> Dict:
    """
    Create a new recordset for a domain.
    https://desec.readthedocs.io/en/latest/dns/rrsets.html#creating-an-rrset

    :param token: The token to authenticate
    :param name: Name of the domain to use
    :param subname: Subdomain to use
    :param type: Record type
    :param ttl: Record TTL
    :param records: List of records
    :return:
    """
    data = {
        'subname': subname,
        'type': type,
        'ttl': ttl,
        'records': records
    }
    r = requests.post(f'https://desec.io/api/v1/domains/{ name }/rrsets/',
                      headers={'Authorization': f'Token { token }'}, json=data)

    if r.status_code == 400:
        raise Exception('Operation cannot be performed with the given parameters')
    elif r.status_code != 201:
        raise Exception('Unknown error')

    return r.json()


# TODO: Bulk Creation of RRsets https://gitlab.com/XenGi/desec-dns-cli/issues/2
# https://desec.readthedocs.io/en/latest/dns/rrsets.html#bulk-creation-of-rrsets


def get_rrsets(token: str, name: str) -> List[Dict]:
    """
    Get all record sets in a zone.
    https://desec.readthedocs.io/en/latest/dns/rrsets.html#retrieving-all-rrsets-in-a-zone

    :param token: The token to authenticate
    :param name: Name of the domain to use
    :return:
    """
    # TODO: handle pagination https://gitlab.com/XenGi/desec-dns-cli/issues/3
    # https://desec.readthedocs.io/en/latest/dns/rrsets.html#pagination
    r = requests.get(f'https://desec.io/api/v1/domains/{ name }/rrsets/', headers={'Authorization': f'Token { token }'})

    if r.status_code != 200:
        raise Exception('Unknown error')

    return r.json()


def get_rrsets_by_type(token: str, name: str, type: str) -> List[Dict]:
    """
    Retrieve an array of all RRsets from your zone that have a specific type.
    https://desec.readthedocs.io/en/latest/dns/rrsets.html#filtering-by-record-type

    :param token: The token to authenticate
    :param name: Name of the domain to use
    :param type: Type to filter for
    :return:
    """
    r = requests.get(f'https://desec.io/api/v1/domains/{ name }/rrsets/?type={ type }',
                     headers={'Authorization': f'Token { token }'})

    if r.status_code != 200:
        raise Exception('Unknown error')

    return r.json()


def get_rrsets_by_subname(token: str, name: str, subname: str) -> List[Dict]:
    """
    Retrieve an array of all RRsets from your zone that have a specific subname.
    https://desec.readthedocs.io/en/latest/dns/rrsets.html#filtering-by-subname

    :param token: The token to authenticate
    :param name: Name of the domain to use
    :param subname: Subname to filter for
    :return:
    """
    r = requests.get(f'https://desec.io/api/v1/domains/{ name }/rrsets/?subname={ subname }',
                     headers={'Authorization': f'Token { token }'})

    if r.status_code == 404:
        raise Exception('RRset not found')
    elif r.status_code != 200:
        raise Exception('Unknown error')

    return r.json()


def get_rrset(token: str, name: str, subname: str, type: str) -> Dict:
    """
    Retrieve a RRset with a specific name and type from your zone.
    https://desec.readthedocs.io/en/latest/dns/rrsets.html#retrieving-a-specific-rrset

    :param token: The token to authenticate
    :param name: Name of the domain to use
    :param subname: Subname to filter for
    :param type: Type to filter for
    :return:
    """
    r = requests.get(f'https://desec.io/api/v1/domains/{ name }/rrsets/{ subname }/{ type }/',
                     headers={'Authorization': f'Token { token }'})

    if r.status_code != 200:
        raise Exception('Unknown error')

    return r.json()


def modify_rrset(token: str, name: str, subname: str, type: str, ttl: int = None, records: List[str] = None) -> Dict:
    """
    Change the ttl or records of a Record Set.
    https://desec.readthedocs.io/en/latest/dns/rrsets.html#modifying-an-rrset

    :param token: The token to authenticate
    :param name: Name of the domain to use
    :param subname: Subname to filter for
    :param type: Type to filter for
    :param ttl: TTL of the RRset
    :param records: List of records in the RRset
    :return:
    """
    data = {}
    if ttl:
        data['ttl'] = ttl
    if records:
        data['records'] = records
        # TXT records must be enclosed in double quotes
        # https://desec.readthedocs.io/en/latest/dns/rrsets.html#caveats
        if type == 'TXT':
            data['records'] = [f'"{ r }"' for r in records]
    r = requests.patch(f'https://desec.io/api/v1/domains/{ name }/rrsets/{ subname }/{ type }/',
                       headers={'Authorization': f'Token { token }'}, json=data)

    if r.status_code == 400:
        raise Exception('Operation cannot be performed with the given parameters')
    elif r.status_code != 200:
        raise Exception('Unknown error')

    return r.json()


# TODO: Bulk Modification of RRsets https://gitlab.com/XenGi/desec-dns-cli/issues/4
# https://desec.readthedocs.io/en/latest/dns/rrsets.html#bulk-modification-of-rrsets


def delete_rrset(token: str, name: str, subname: str, type: str):
    """
    Delete a RRset.
    https://desec.readthedocs.io/en/latest/dns/rrsets.html#deleting-an-rrset

    :param token: The token to authenticate
    :param name: Name of the domain to use
    :param subname: Subname to filter for
    :param type: Type to filter for
    :return:
    """
    r = requests.delete(f'https://desec.io/api/v1/domains/{ name }/rrsets/{ subname }/{ type }/',
                        headers={'Authorization': f'Token { token }'})

    if r.status_code != 204:
        raise Exception('Unknown error')


# TODO: Bulk Deletion of RRsets https://gitlab.com/XenGi/desec-dns-cli/issues/5
# https://desec.readthedocs.io/en/latest/dns/rrsets.html#bulk-deletion-of-rrsets
