from typing import List, Dict

import requests

from desec_dns_api import UnauthorizedError, UnexpectedError


def get_all_tokens(token: str) -> List[Dict]:
    """
    Retrieves all current tokens.
    https://desec.readthedocs.io/en/latest/auth/tokens.html#retrieving-all-current-tokens

    :param token: The token to authenticate
    :return: A list of tokens
    """
    r = requests.get('https://desec.io/api/v1/auth/tokens/', headers={'Authorization': f'Token { token }'})

    if r.status_code == 401:
        raise UnauthorizedError('Authentication failed')
    elif r.status_code != 200:
        raise UnexpectedError(f'API returned status { r.status_code }')

    # [
    #     {
    #         "created": "2018-09-06T07:05:54.080564Z",
    #         "id": "3159e485-5499-46c0-ae2b-aeb84d627a8e",
    #         "name": "login"
    #     },
    #     {
    #         "created": "2018-09-06T08:53:26.428396Z",
    #         "id": "76d6e39d-65bc-4ab2-a1b7-6e94eee0a534",
    #         "name": ""
    #     }
    # ]
    return r.json()


def create_token(token: str, name: str = "") -> Dict:
    """
    Create a new token for the account.
    https://desec.readthedocs.io/en/latest/auth/tokens.html#create-additional-tokens

    :param token: The token to authenticate
    :param name: Optional name for the token
    :return: The created token
    """
    data = {
        'name': name
    }
    r = requests.post('https://desec.io/api/v1/auth/tokens/', headers={'Authorization': f'Token { token }'}, json=data)

    if r.status_code == 401:
        raise UnauthorizedError('Authentication failed')
    elif r.status_code != 201:
        raise UnexpectedError(f'API returned status { r.status_code }')

    # {
    #     "created": "2018-09-06T09:08:43.762697Z",
    #     "id": "3a6b94b5-d20e-40bd-a7cc-521f5c79fab3",
    #     "token": "4pnk7u+NHvrEkFzrhFDRTjGFyX+S",
    #     "name": "my new token"
    # }
    return r.json()


def delete_token(token: str, id: str):
    """
    Remove a token from the account.
    https://desec.readthedocs.io/en/latest/auth/tokens.html#delete-tokens

    :param token: The token to authenticate
    :param id: The token id to delete
    :return:
    """
    r = requests.delete(f'https://desec.io/api/v1/auth/tokens/{ id }/', headers={'Authorization': f'Token { token }'})

    if r.status_code == 401:
        raise UnauthorizedError('Authentication failed')
    elif r.status_code != 204:
        raise UnexpectedError(f'API returned status { r.status_code }')
